@@ -194,10 +194,6 @@ struct GIFImageReader {
     }
 
     ~GIFImageReader() {
-        close();
-    }
-
-    void close() {
         delete []global_colormap;
         global_colormap = 0;
         delete frame_reader;
@@ -208,8 +204,8 @@ struct GIFImageReader {
               WebCore::GIFImageDecoder::GIFQuery query = WebCore::GIFImageDecoder::GIFFullQuery, unsigned haltAtFrame = -1);
 
 private:
-    void output_row();
-    int do_lzw(const unsigned char *q);
+    bool output_row();
+    bool do_lzw(const unsigned char *q);
 };
 
 #endif
