@@ -116,7 +116,6 @@ protected:
 
     // The following bitfield is 32-bits long, which optimizes padding with the
     // int refCount in the base class. Beware when adding more bits.
-    unsigned m_pseudoState : 3; // PseudoState
     bool m_affectedByAttributeSelectors : 1;
     bool m_unique : 1;
 
@@ -133,7 +132,7 @@ protected:
     bool m_childrenAffectedByBackwardPositionalRules : 1;
     bool m_firstChildState : 1;
     bool m_lastChildState : 1;
-    unsigned m_childIndex : 18; // Plenty of bits to cache an index.
+    unsigned m_childIndex : 21; // Plenty of bits to cache an index.
 
     // non-inherited attributes
     DataRef<StyleBoxData> box;
@@ -176,7 +175,8 @@ protected:
                    (_visuallyOrdered == other._visuallyOrdered) &&
                    (_htmlHacks == other._htmlHacks) &&
                    (_force_backgrounds_to_white == other._force_backgrounds_to_white) &&
-                   (_pointerEvents == other._pointerEvents);
+                   (_pointerEvents == other._pointerEvents) &&
+                   (_insideLink == other._insideLink);
         }
 
         bool operator!=(const InheritedFlags& other) const { return !(*this == other); }
@@ -201,7 +201,8 @@ protected:
         bool _htmlHacks : 1;
         bool _force_backgrounds_to_white : 1;
         unsigned _pointerEvents : 4; // EPointerEvents
-        // 41 bits
+        EInsideLink _insideLink : 2; // Whether or not we are contained inside a link.
+        // 43 bits
     } inherited_flags;
 
 // don't inherit
@@ -225,7 +226,8 @@ protected:
                 && _affectedByActive == other._affectedByActive
                 && _affectedByDrag == other._affectedByDrag
                 && _pseudoBits == other._pseudoBits
-                && _unicodeBidi == other._unicodeBidi;
+                && _unicodeBidi == other._unicodeBidi
+                && _isLink == other._isLink;
         }
 
         bool operator!=(const NonInheritedFlags& other) const { return !(*this == other); }
@@ -244,12 +246,13 @@ protected:
         unsigned _page_break_after : 2; // EPageBreak
         unsigned _page_break_inside : 2; // EPageBreak
 
-        unsigned _styleType : 5; // PseudoId
+        unsigned _styleType : 6; // PseudoId
         bool _affectedByHover : 1;
         bool _affectedByActive : 1;
         bool _affectedByDrag : 1;
         unsigned _pseudoBits : 7;
         unsigned _unicodeBidi : 2; // EUnicodeBidi
+        bool _isLink : 1;
         // 50 bits
     } noninherited_flags;
 
@@ -275,6 +278,7 @@ protected:
         inherited_flags._box_direction = initialBoxDirection();
         inherited_flags._force_backgrounds_to_white = false;
         inherited_flags._pointerEvents = initialPointerEvents();
+        inherited_flags._insideLink = NotInsideLink;
 
         noninherited_flags._effectiveDisplay = noninherited_flags._originalDisplay = initialDisplay();
         noninherited_flags._overflowX = initialOverflowX();
@@ -293,6 +297,7 @@ protected:
         noninherited_flags._affectedByDrag = false;
         noninherited_flags._pseudoBits = 0;
         noninherited_flags._unicodeBidi = initialUnicodeBidi();
+        noninherited_flags._isLink = false;
     }
 
 protected:
@@ -350,6 +355,7 @@ public:
 
     bool isStyleAvailable() const;
 
+    bool hasAnyPublicPseudoStyles() const;
     bool hasPseudoStyle(PseudoId pseudo) const;
     void setHasPseudoStyle(PseudoId pseudo);
 
@@ -586,6 +592,9 @@ public:
 
     CursorList* cursors() const { return inherited->cursorData.get(); }
 
+    EInsideLink insideLink() const { return static_cast<EInsideLink>(inherited_flags._insideLink); }
+    bool isLink() const { return noninherited_flags._isLink; }
+
     short widows() const { return inherited->widows; }
     short orphans() const { return inherited->orphans; }
     EPageBreak pageBreakInside() const { return static_cast<EPageBreak>(noninherited_flags._page_break_inside); }
@@ -911,6 +920,9 @@ public:
     void setCursorList(PassRefPtr<CursorList>);
     void clearCursorList();
 
+    void setInsideLink(EInsideLink insideLink) { inherited_flags._insideLink = insideLink; }
+    void setIsLink(bool b) { noninherited_flags._isLink = b; }
+
     bool forceBackgroundsToWhite() const { return inherited_flags._force_backgrounds_to_white; }
     void setForceBackgroundsToWhite(bool b=true) { inherited_flags._force_backgrounds_to_white = b; }
 
@@ -1068,10 +1080,6 @@ public:
                originalDisplay() == INLINE_BOX || originalDisplay() == INLINE_TABLE;
     }
 
-    // To obtain at any time the pseudo state for a given link.
-    PseudoState pseudoState() const { return static_cast<PseudoState>(m_pseudoState); }
-    void setPseudoState(PseudoState s) { m_pseudoState = s; }
-
     // To tell if this style matched attribute selectors. This makes it impossible to share.
     bool affectedByAttributeSelectors() const { return m_affectedByAttributeSelectors; }
     void setAffectedByAttributeSelectors() { m_affectedByAttributeSelectors = true; }
@@ -1101,6 +1109,8 @@ public:
     unsigned childIndex() const { return m_childIndex; }
     void setChildIndex(unsigned index) { m_childIndex = index; }
 
+    Color visitedDependentColor(int colorProperty) const;
+
     // Initial values for all the properties
     static bool initialBorderCollapse() { return false; }
     static EBorderStyle initialBorderStyle() { return BNONE; }
