@@ -35,21 +35,22 @@
 
 namespace WebCore {
 
-    class SecurityOrigin;
-    class StorageArea;
+class Page;
+class SecurityOrigin;
+class StorageArea;
 
-    // This interface is required for Chromium since these actions need to be proxied between processes.
-    class StorageNamespace : public RefCounted<StorageNamespace> {
-    public:
-        static PassRefPtr<StorageNamespace> localStorageNamespace(const String& path);
-        static PassRefPtr<StorageNamespace> sessionStorageNamespace();
+// This interface is required for Chromium since these actions need to be proxied between processes.
+class StorageNamespace : public RefCounted<StorageNamespace> {
+public:
+    static PassRefPtr<StorageNamespace> localStorageNamespace(const String& path, unsigned quota);
+    static PassRefPtr<StorageNamespace> sessionStorageNamespace(Page*);
 
-        virtual ~StorageNamespace() { }
-        virtual PassRefPtr<StorageArea> storageArea(SecurityOrigin*) = 0;
-        virtual PassRefPtr<StorageNamespace> copy() = 0;
-        virtual void close() = 0;
-        virtual void unlock() = 0;
-    };
+    virtual ~StorageNamespace() { }
+    virtual PassRefPtr<StorageArea> storageArea(PassRefPtr<SecurityOrigin>) = 0;
+    virtual PassRefPtr<StorageNamespace> copy() = 0;
+    virtual void close() = 0;
+    virtual void unlock() = 0;
+};
 
 } // namespace WebCore
 
